@@ -12,7 +12,7 @@ package MooX::Options::Role;
 use strict;
 use warnings;
 
-our $VERSION = '4.003';    # VERSION
+our $VERSION = '4.004';    # VERSION
 
 use MRO::Compat;
 use MooX::Options::Descriptive;
@@ -28,15 +28,7 @@ use Scalar::Util qw/blessed/;
 
 sub _option_name {
     my ( $name, %data ) = @_;
-    my $cmdline_name = $name;
-    $cmdline_name .= '|' . $data{short} if defined $data{short};
-
-    #dash name support
-    my $dash_name = $name;
-    $dash_name =~ tr/_/-/;
-    if ( $dash_name ne $name ) {
-        $cmdline_name .= '|' . $dash_name;
-    }
+    my $cmdline_name = join( '|', grep {defined} ( $name, $data{short} ) );
     $cmdline_name .= '+' if $data{repeatable} && !defined $data{format};
     $cmdline_name .= '!' if $data{negativable};
     $cmdline_name .= '=' . $data{format} if defined $data{format};
@@ -64,14 +56,14 @@ sub _options_prepare_descriptive {
         push @options, [ _option_name( $name, %data ), $doc ];
 
         if ( defined $data{autosplit} ) {
-            $has_to_split{"--${name}"} = Data::Record->new(
+            $has_to_split{$name} = Data::Record->new(
                 { split => $data{autosplit}, unless => $RE{quoted} } );
             if ( my $short = $data{short} ) {
-                $has_to_split{"-${short}"} = $has_to_split{"--${name}"};
+                $has_to_split{$short} = $has_to_split{ ${name} };
             }
-            for ( my $i = 1; $i < length($name); $i++ ) {
+            for ( my $i = 1; $i <= length($name); $i++ ) {
                 my $long_short = substr( $name, 0, $i );
-                $has_to_split{"--${long_short}"} = $has_to_split{"--${name}"};
+                $has_to_split{$long_short} = $has_to_split{ ${name} };
             }
         }
     }
@@ -79,19 +71,51 @@ sub _options_prepare_descriptive {
     return \@options, \%has_to_split;
 }
 
-sub _options_split_with {
-    my ($has_to_split) = @_;
+sub _options_fix_argv {
+    my ( $option_data, $has_to_split ) = @_;
 
     my @new_argv;
 
     #parse all argv
-    for my $i ( 0 .. $#ARGV ) {
-        my $arg = $ARGV[$i];
-        my ( $arg_name, $arg_values ) = split( /=/x, $arg, 2 );
-        unless ( defined $arg_values ) {
-            $arg_values = $ARGV[ ++$i ];
+    while ( my $arg = shift @ARGV ) {
+        if ( $arg eq '--' ) {
+            push @new_argv, $arg, @ARGV;
+            last;
         }
-        if ( my $rec = $has_to_split->{$arg_name} ) {
+        if ( index( $arg, '-' ) != 0 ) {
+            push @new_argv, $arg;
+            next;
+        }
+
+        my ( $arg_name_with_dash, $arg_values ) = split( /=/x, $arg, 2 );
+        if ( index( $arg_name_with_dash, '--' ) < 0 && !defined $arg_values )
+        {
+            $arg_values = substr( $arg_name_with_dash, 2 );
+            $arg_name_with_dash = substr( $arg_name_with_dash, 0, 2 );
+        }
+        unshift @ARGV, $arg_values if defined $arg_values;
+
+        my ( $dash, $negative, $arg_name_without_dash )
+            = $arg_name_with_dash =~ /^(\-+)(no\-)?(.*)$/x;
+        $arg_name_without_dash =~ s/\-/_/gx;
+
+        my $arg_name = $dash;
+
+        if ( defined $negative ) {
+            if ( exists $option_data->{$arg_name_without_dash}
+                && $option_data->{$arg_name_without_dash}{negativable} )
+            {
+                $arg_name .= 'no-';
+            }
+            else {
+                $arg_name .= 'no_';
+            }
+        }
+
+        $arg_name .= $arg_name_without_dash;
+
+        if ( my $rec = $has_to_split->{$arg_name_without_dash} ) {
+            $arg_values = shift @ARGV;
             foreach my $record ( $rec->records($arg_values) ) {
 
                 #remove the quoted if exist to chain
@@ -100,7 +124,7 @@ sub _options_split_with {
             }
         }
         else {
-            push @new_argv, $arg;
+            push @new_argv, $arg_name;
         }
     }
 
@@ -190,7 +214,7 @@ sub parse_options {
         = _options_prepare_descriptive( \%options_data );
 
     local @ARGV = @ARGV if $options_config{protect_argv};
-    @ARGV = _options_split_with($has_to_split) if %$has_to_split;
+    @ARGV = _options_fix_argv( \%options_data, $has_to_split );
 
     my @flavour;
     if ( defined $options_config{flavour} ) {
@@ -351,7 +375,7 @@ MooX::Options::Role - role that is apply to your object
 
 =head1 VERSION
 
-version 4.003
+version 4.004
 
 =head1 METHODS
 
